package com.skica.spvbpisvsportalbackend;

import com.skica.spvbpisvsportalbackend.entity.unused.Role;
import jakarta.persistence.NoResultException;
import jakarta.persistence.criteria.CriteriaBuilder;
import jakarta.persistence.criteria.CriteriaQuery;
import jakarta.persistence.criteria.Root;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;


/**
 * Initializes default roles (if not present) on startup.
 */
@Component
public class DataInitializationOnStartup implements ApplicationListener<ApplicationReadyEvent> {
    /**
     * ISO: Information Security Officer
     */
    @Autowired
    protected SessionFactory sessionFactory;

    @Override
    public void onApplicationEvent(ApplicationReadyEvent event) {
        Session session = sessionFactory.openSession();

        // on db init?
        CriteriaBuilder cb = session.getCriteriaBuilder();
        CriteriaQuery<Role> cq = cb.createQuery(Role.class);
        Root<Role> root = cq.from(Role.class);
        cq.select(root).where(cb.equal(root.get("name"), Role.ISO_NAME));

        try {
            session.createQuery(cq).getSingleResult();
        }
        catch (NoResultException ex) {
            Transaction tx = session.beginTransaction();
            session.persist(Role.ISO());
            tx.commit();
        }

        session.close();
    }
}
