package com.skica.spvbpisvsportalbackend.controller;

import com.skica.spvbpisvsportalbackend.entity.business.BusinessProcess;
import com.skica.spvbpisvsportalbackend.entity.business.DescribedProtection;
import com.skica.spvbpisvsportalbackend.enums.BasicProtectionLevel;
import com.skica.spvbpisvsportalbackend.enums.BasicProtectionValue;
import com.skica.spvbpisvsportalbackend.repository.BusinessProcessRepository;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.StreamSupport;

@Controller
@RequestMapping("/svc/business-process")
public class BusinessProcessController {
    @Autowired
    private BusinessProcessRepository businessProcessRepository;


    @GetMapping({"/view/{entityId}"})
    public String view(@PathVariable("entityId") Long entityId, Model model) {
        var system = businessProcessRepository.findById(entityId);
        if(system.isEmpty())
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "System " + entityId + " does not exists.");
        model.addAttribute("system", system.get());

        return "view/system";
    }

    @PostMapping({"/delete/{entityId}"})
    public String delete(@PathVariable("entityId") Long entityId, final HttpServletRequest httpServletRequest) {
        businessProcessRepository.deleteById(entityId);
        return "redirect:" + httpServletRequest.getHeader(HttpHeaders.REFERER);
    }

    @PostMapping(value="/create", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public String save(final BusinessProcess businessProcess,
                     final BindingResult bindingResult,
                     final ModelMap model,
                     final HttpServletRequest httpServletRequest) {
        if (bindingResult.hasErrors()) {
            throw new RuntimeException(String.valueOf(bindingResult.getAllErrors()));
        }

        businessProcessRepository.save(businessProcess);
        model.clear();

        return "redirect:" + httpServletRequest.getHeader(HttpHeaders.REFERER);
    }

    @PostMapping({"/{entityId}/describe-protection"})
    public ResponseEntity<Void> delete(@PathVariable("entityId") final Long entityId,
                                 @RequestParam Map<String, Object> requestParams) {
        var entityOpt = businessProcessRepository.findById(entityId);
        if(entityOpt.isEmpty()) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "System " + entityId + " does not exists.");
        } 
        var entity = entityOpt.get();
        
        var descriptionProtection = entity.getDescribedProtection(BasicProtectionValue.values()[Integer.valueOf((String)requestParams.get("value"))]);
        descriptionProtection.setLevel(BasicProtectionLevel.values()[Integer.valueOf((String)requestParams.get("level"))]);
        descriptionProtection.setDescription((String)requestParams.get("description"));

        businessProcessRepository.save(entity);
        
        return ResponseEntity.noContent().build();
    }

    @PostMapping({"/set-main"})
    public ResponseEntity<Void> setMain(@RequestParam(value = "selectedBusinessProcesses", required = false) List<Long> selectedBusinessProcesses) {
        // can we do this more elegantly?
        List<BusinessProcess> bps;
        if(selectedBusinessProcesses != null) {
            bps = StreamSupport.stream(businessProcessRepository.findAll().spliterator(), false)
                    .peek(bp -> bp.isMain = selectedBusinessProcesses.contains(bp.id))
                    .toList();
        } else {
            bps = StreamSupport.stream(businessProcessRepository.findAll().spliterator(), false)
                    .peek(bp -> bp.isMain = false)
                    .toList();
        }

        businessProcessRepository.saveAll(bps);

        return ResponseEntity.noContent().build();
    }
}
