package com.skica.spvbpisvsportalbackend.controller;

import com.skica.spvbpisvsportalbackend.entity.OrganisationApplication;
import com.skica.spvbpisvsportalbackend.repository.BusinessProcessRepository;
import com.skica.spvbpisvsportalbackend.repository.OrganisationApplicationRepository;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.stream.StreamSupport;

@Controller
@RequestMapping("/svc/organisation-application")
public class OrganisationApplicationController {
    @Autowired
    private OrganisationApplicationRepository organisationApplicationRepository;

    @Autowired
    private BusinessProcessRepository businessProcessRepository;


    @GetMapping({"/view/{entityId}"})
    public String view(@PathVariable("entityId") String entityId, Model model) {
        var system = organisationApplicationRepository.findById(entityId);
        if(system.isEmpty())
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Organisation application " + entityId + " does not exists.");
        model.addAttribute("organisationApplication", system.get());

        return "view/organisationApplication";
    }

    @PostMapping(value="/delete/{entityCode}", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public String delete(
            final @PathVariable("entityCode") String entityCode,
            final HttpServletRequest httpServletRequest) {
        organisationApplicationRepository.deleteById(entityCode);
        return "redirect:" + httpServletRequest.getHeader(HttpHeaders.REFERER);
    }

    @PostMapping(value="/create", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public String save(final @ModelAttribute("createOrganisationApplication") OrganisationApplication organsationApplication,
                       final @RequestParam(value = "selectedBusinessProcesses", required = false) List<Long> selectedBusinessProcesses,
                       final BindingResult bindingResult,
                       final HttpServletRequest httpServletRequest) {
        if (bindingResult.hasErrors()) {
            throw new RuntimeException(String.valueOf(bindingResult.getAllErrors()));
        }

        if(selectedBusinessProcesses != null) {
            organsationApplication.presentInBusinessProcesses = StreamSupport.stream(businessProcessRepository.findAllById(selectedBusinessProcesses).spliterator(), false).toList();
        }
        if(organsationApplication.code != null && !organsationApplication.code.isEmpty()) {
            organisationApplicationRepository.save(organsationApplication);
        }

        return "redirect:" + httpServletRequest.getHeader(HttpHeaders.REFERER);
    }
}
