package com.skica.spvbpisvsportalbackend.controller;

import com.skica.spvbpisvsportalbackend.Common;
import com.skica.spvbpisvsportalbackend.DictionaryProvider;
import com.skica.spvbpisvsportalbackend.entity.*;
import com.skica.spvbpisvsportalbackend.entity.business.BusinessProcess;
import com.skica.spvbpisvsportalbackend.entity.business.DescribedProtection;
import com.skica.spvbpisvsportalbackend.entity.isms.*;
import com.skica.spvbpisvsportalbackend.entity.unused.Asset;
import com.skica.spvbpisvsportalbackend.entity.unused.Employee;
import com.skica.spvbpisvsportalbackend.entity.unused.Role;
import com.skica.spvbpisvsportalbackend.enums.*;
import com.skica.spvbpisvsportalbackend.repository.BusinessProcessRepository;
import com.skica.spvbpisvsportalbackend.repository.OrganisationApplicationRepository;
import com.skica.spvbpisvsportalbackend.repository.OrganisationItSystemRepository;
import com.skica.spvbpisvsportalbackend.repository.OrganisationRoomRepository;
import com.skica.spvbpisvsportalbackend.workflow.service.IsmsSetupService;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@Controller
@RequestMapping(path = "/")
public class UIController {
    @Autowired
    private SessionFactory sessionFactory;

    @Autowired
    private DictionaryProvider dictionary;

    @Autowired
    private IsmsSetupService ismsSetupService;

    @Autowired
    private BusinessProcessRepository businessProcessRepository;

    @Autowired
    private OrganisationApplicationRepository organisationApplicationRepository;

    @Autowired
    private OrganisationItSystemRepository organisationItSystemRepository;

    @Autowired
    private OrganisationRoomRepository organisationRoomRepository;

    @ModelAttribute("allBusinessProcesses")
    public List<BusinessProcess> allBusinessProcesses() {
        return Common.getEntities(sessionFactory, BusinessProcess.class);
    }
    @ModelAttribute("allEmployees")
    public List<Employee> allEmployees() {
        return Common.getEntities(sessionFactory, Employee.class);
    }
    @ModelAttribute("allAssets")
    public List<Asset> allAssets() {
        return Common.getEntities(sessionFactory, Asset.class);
    }
    @ModelAttribute("allRoles")
    public List<Role> allRoles() { return Common.getEntities(sessionFactory, Role.class);
    }
    @ModelAttribute("allOrganisationApplications")
    public List<OrganisationApplication> allOrganisationApplications() {
        return Common.getEntities(sessionFactory, OrganisationApplication.class);
    }
    @ModelAttribute("allOrganisationItSystems")
    public List<OrganisationItSystem> allOrganisationItSystems() {
        return Common.getEntities(sessionFactory, OrganisationItSystem.class);
    }
    @ModelAttribute("allOrganisationRooms")
    public List<OrganisationRoom> allOrganisationRooms() {
        return Common.getEntities(sessionFactory, OrganisationRoom.class);
    }

    @GetMapping({"/_layout.html"})
    public String page() {
        return "_layout.html";
    }
    @GetMapping({"/", "/index.html"})
    public String index() {
        return "index.html";
    }
    @GetMapping({"/dictionary"})
    public String dictionary(ModelMap modelMap) {
        modelMap.addAttribute("dictionaryTerms", dictionary.dictionaryTerms);
        return  "dictionary";
    }
    @GetMapping({"/isms-setup"})
    public String ismsSetup() {
        return  "isms-setup/index";
    }

    @GetMapping({"/isms-setup/{step}"})
    public String ismsSetup(@PathVariable("step") String step, Model model) {
        model.addAttribute("ismsSetupModel", ismsSetupService.getIsmsSetupModel());
        model.addAttribute("createBusinessProcess", new BusinessProcess());
        model.addAttribute("createSecurityObjective", new SecurityObjective());
        model.addAttribute("createOrganisationApplication", new OrganisationApplication());
        model.addAttribute("createOrganisationItSystem", new OrganisationItSystem());
        model.addAttribute("createOrganisationRoom", new OrganisationRoom());
        model.addAttribute("basicProtectionValues", BasicProtectionValue.values());
        model.addAttribute("basicProtectionLevels", BasicProtectionLevel.values());
        model.addAttribute("ismsSetupModel", ismsSetupService.getIsmsSetupModel());

        switch (step) {
            default -> {}
        }
        return  "isms-setup/" + step;
    }

    @GetMapping({"/isms-setup/security-measures/{step}"})
    public String securityMeasures(@PathVariable("step") String step, Model model) {
        model.addAttribute("ismsSetupModel", ismsSetupService.getIsmsSetupModel());
        return  "isms-setup/security-measures/" + step;
    }

    @GetMapping({"/isms-check"})
    public String ismsCheck() {
        return "isms-check";
    }
    @GetMapping({"/isms-check/", "/isms-check/{path}"})
    public String ismsCheck(@PathVariable(name="path", required = false) String path) {
        return "isms-check/" + (path == null ? "index" : path);
    }

    @GetMapping({"/list"})
    public String lists() {
        return "lists/index";
    }
    @GetMapping({"/list/{entity}"})
    public String listEntity(@PathVariable("entity") String entity, Model model) {
        model.addAttribute("ismsSetupModel", ismsSetupService.getIsmsSetupModel());
        return "lists/" + entity;
    }

    @GetMapping({"/create/{entity}"})
    public String createEntity(@PathVariable("entity") String entity, Model model) {
        switch (entity) { case "role" -> model.addAttribute("createRole", new Role());
            case "asset" -> model.addAttribute("createAsset", new Asset());
            case "system" -> model.addAttribute("createSystem", new BusinessProcess());
            case "employee" ->{
                model.addAttribute("createEmployee", new Employee());
                model.addAttribute("selectedRoles", new ArrayList<Role>());
            }
                default -> throw new IllegalStateException("Unexpected value: " + entity);
        }

        return "create/" + entity;
    }

    @GetMapping({"/view/{entity}/{id}"})
    public String view(@PathVariable("entity") final String entity,  @PathVariable("id") final String id, final Model model) {
        try (var session = sessionFactory.openSession()) {
            switch (entity) {
                case "role" -> model.addAttribute("role", session.get(Role.class, id));
                case "asset" -> model.addAttribute("asset", session.get(Asset.class, id));
                case "employee" -> model.addAttribute("employee", session.get(Employee.class, id));
                case "business-process" -> model.addAttribute("businessProcess", session.get(BusinessProcess.class, id));
                default -> throw new IllegalStateException("Unexpected value: " + entity);
            }
        }
        return "view/" + entity;
    }

    @GetMapping({"/legislation"})
    public String legislation() {
        return "legislation";
    }

    @PostMapping("/setup-with-example-data")
    public ResponseEntity<Void> fillWithExampleData(){
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();

        var officialInformation = new OfficialInformation();
        ismsSetupModel.setOfficialInformation(officialInformation);
        officialInformation.setOrganisationName("Obecný úrad Bubble-bum");
        officialInformation.setOrganisationNameAcronym("OÚB");
        officialInformation.setOrganisationCharacteristics("obecný úrad");
        officialInformation.setOrganisationFoundation("369/1990 Zb. o obecnom zriadení");
        officialInformation.setOrganisationTasks("spravujeme matriku, schvaľujeme rozpočet");

        var frameworkConditions = new FrameworkConditions();
        ismsSetupModel.setFrameworkConditions(frameworkConditions);
        frameworkConditions.setOrganisationGoals("""
                Našimi cieľmi sú spracovávanie osobných údajov v matrike.
                """);
        frameworkConditions.setOrganisationStructure("""
                Máme starostu, sekretárku a dvoch informatikov.
                Starosta nás riadi.
                Sekretárka zodpovedá za dokumenty a inventarizáciu.
                Informatici spravujú IT systémy.
                """);
        frameworkConditions.setOrganisationThirdParties("""
                Pre jednoduchosť, nemáme externých dodávateľov.
                """);
        frameworkConditions.setOrganisationThirdParties("""
                Pre jednoduchosť, nemáme externých dodávateľov.
                """);
        frameworkConditions.setOrganisationStrategicContext("""
                Nemáme potrebu podstupovať riziko, s nikým nesúťažíme, naopak spolupracujeme s ďalšími úradmi.
                """);


        ismsSetupModel.setSecurityObjectives(new ArrayList<>(List.of(
                new SecurityObjective("Vysoká spoľahlivosť a kvalita spracovávania informácie.", List.of(BasicProtectionValue.INTEGRITY, BasicProtectionValue.AVAILABILITY, BasicProtectionValue.CONFIDENTIALITY)),
                new SecurityObjective("Ochrana fyzických osôb, ich zdravie a osobné údaje.", List.of())
        )));

        ismsSetupModel.setSecondaryAssets(new ArrayList<>(List.of(GeneralSecondaryAsset.INFORMATION, GeneralSecondaryAsset.EMPLOYEES)));

        var threatAndRiskOne = new ThreatAndRisk();
        threatAndRiskOne.setThreatType(ThreatType.EMPLOYEES_HUMAN_ERRORS_AND_MISTAKES);
        threatAndRiskOne.calculateThreatLevel(ThreatProbability.OFTEN, ThreatImpact.CRITICAL);
        var threatAndRiskTwo = new ThreatAndRisk();
        threatAndRiskTwo.setThreatType(ThreatType.THEFT);
        threatAndRiskTwo.calculateThreatLevel(ThreatProbability.NEGLIGIBLE, ThreatImpact.CONSIDERABLE);
        ismsSetupModel.setThreatsAndRisks(new ArrayList<>(List.of(threatAndRiskOne, threatAndRiskTwo)));

        ismsSetupModel.setAcceptableRiskLevel(RiskLevel.MEDIUM);
        ismsSetupModel.setGuarantorKIB("Starosta Mako");

        ismsSetupModel.setSecurityPrinciples(new ArrayList<>(List.of(
                SecurityPrinciple.USER_IDENTIFICATION_AND_AUTHENTIFICATION,
                SecurityPrinciple.NEED_TO_KNOW,
                SecurityPrinciple.NEED_TO_USE,
                SecurityPrinciple.LEAST_PRIVILEGE)));

        var formalData = new FormalData();
        ismsSetupModel.setFormalData(formalData);
        formalData.setStatutory("Starosta Mako");
        formalData.setOrganizationalLeadershipStatus(OrganisationalLeadershipStatus.COLLECTIVE_MANAGEMENT);
        formalData.setManagingAuthority("starosta");
        formalData.setDocumentsRelease("elektronicky");
        formalData.setResponsibleForEconomy("Sekretárka Evička Jablková");
        formalData.setResponsibleForIT("Informatik Janko Dub");
        formalData.setDocumentsKIB(new ArrayList<>(List.of("Prevádzkový poriadok IT", "Výsledky auditov")));

        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        organisationApplicationRepository.deleteAll();
        organisationItSystemRepository.deleteAll();
        organisationRoomRepository.deleteAll();
        businessProcessRepository.deleteAll();

        var businessProcess = new BusinessProcess();
        businessProcess.setMain(true);
        businessProcess.setName("Matrika");
        businessProcess.setProcessedInformation("Osobné údaje");
        businessProcess.setOwner("Starosta Janko");
        businessProcess.setDescribedProtection(List.of(
                new DescribedProtection(BasicProtectionValue.CONFIDENTIALITY, BasicProtectionLevel.HIGH, "Sú prítomné osobné údaje."),
                new DescribedProtection(BasicProtectionValue.INTEGRITY, BasicProtectionLevel.HIGH, "Osobné údaje musia byť správne."),
                new DescribedProtection(BasicProtectionValue.AVAILABILITY, BasicProtectionLevel.BASIC, "Čakanie nevadí.")
        ));
        businessProcessRepository.save(businessProcess);

        var applicationOne = new OrganisationApplication();
        applicationOne.setCode("A001");
        applicationOne.setName("Microsoft Office");
        applicationOne.setDescription("Kancelársky balík - Word, Excel, PowerPoint");
        applicationOne.setCount(12);
        applicationOne.setUsers("Všetci používatelia");
        applicationOne.setResponsible("IT oddelenie");
        applicationOne.presentInBusinessProcesses = List.of(businessProcess);

        var applicationTwo = new OrganisationApplication();
        applicationTwo.setCode("A002");
        applicationTwo.setName("Active Directory");
        applicationTwo.setDescription("Správa používateľov a prístupov");
        applicationTwo.setCount(1);
        applicationTwo.setUsers("Administrátor");
        applicationTwo.setResponsible("IT oddelenie");
        applicationTwo.presentInBusinessProcesses = List.of(businessProcess);

        organisationApplicationRepository.saveAll(List.of(applicationOne, applicationTwo));

        var itSystemOne = new OrganisationItSystem();
        itSystemOne.setCode("P001");
        itSystemOne.setName("Počítač");
        itSystemOne.setDescription("Stolný počítač");
        itSystemOne.setLocation("kancelária");
        itSystemOne.setCount(2);
        itSystemOne.setUsers("starosta. sekretárka");
        itSystemOne.setResponsible("IT administrátor");

        var itSystemTwo = new OrganisationItSystem();
        itSystemTwo.setCode("S002");
        itSystemTwo.setName("Server");
        itSystemTwo.setDescription("Server s dátami");
        itSystemTwo.setLocation("serverová miestnosť");
        itSystemTwo.setCount(1);
        itSystemTwo.setUsers("všetci");
        itSystemTwo.setResponsible("IT administrátor");
        organisationItSystemRepository.saveAll(List.of(itSystemOne, itSystemTwo));

        var roomOne = new OrganisationRoom();
        roomOne.setCode("B001");
        roomOne.setName("Obecná úrad Bablgam");
        roomOne.setLocation("Obec Bablgam");
        roomOne.setType("budova");

        var roomTwo = new OrganisationRoom();
        roomTwo.setCode("M002");
        roomTwo.setName("Kancelária");
        roomTwo.setLocation("B001");
        roomTwo.setType("miestnosť");
        organisationRoomRepository.saveAll(List.of(roomOne, roomTwo));

        return ResponseEntity.ok().build();
    }
}
