package com.skica.spvbpisvsportalbackend.controller.unused;

import com.skica.spvbpisvsportalbackend.Common;
import com.skica.spvbpisvsportalbackend.entity.unused.Employee;
import com.skica.spvbpisvsportalbackend.entity.unused.Role;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Controller
@RequestMapping("/svc/employee")
public class EmployeeController {
    @Autowired
    private SessionFactory sessionFactory;

    @PostMapping(value = "{employeeId}/bind/role/{roleId}")
    ResponseEntity<String> bindRole(@PathVariable long employeeId, @PathVariable long roleId) {
        Session session = sessionFactory.openSession();
        Transaction tx = session.beginTransaction();
        var employee = session.get(Employee.class, employeeId);
        if(employee == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("No employee with id" + employeeId);
        }

        var role = session.get(Role.class, roleId);
        if(role == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("No role with id" + roleId);
        }
        employee.roles.add(role);
        tx.commit();
        session.close();

        return ResponseEntity.status(HttpStatus.CREATED).body("Successfully bound");
    }

    @PostMapping(value="/create")
    public String save(
            @ModelAttribute("employee") final Employee entity,
            @RequestParam("selectedRoles") final List<Long> selectedRoles,
            final BindingResult bindingResult,
            final ModelMap model) {
        if (bindingResult.hasErrors()) {
            return "/";
        }

        entity.roles = Common.getEntities(sessionFactory, Role.class).stream().filter(r -> selectedRoles.contains(r.name)).collect(Collectors.toSet());
        try (var session = sessionFactory.openSession()) {
            var tx = session.beginTransaction();
            session.persist(entity);
            tx.commit();
        }
        model.clear();
        return "redirect:/view/employee/" + entity.firstName;
    }
    @PostMapping(value = "{employeeName}/unbind/role/{roleName}")
    ResponseEntity<String> unbindRole(@PathVariable String employeeName, @PathVariable String roleName) {
        Session session = sessionFactory.openSession();
        Transaction tx = session.beginTransaction();
        var employee = session.get(Employee.class, employeeName);
        if(employee == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("No employee with id" + employeeName);
        }
        Optional<Role> removeRole = employee.roles.stream().filter(role -> role.name == roleName).findAny();
        if(removeRole.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("No role with id" + roleName);
        }
        employee.roles.remove(removeRole.get());
        tx.commit();
        session.close();

        return ResponseEntity.status(HttpStatus.CREATED).body("Successfully unbound");
    }
    /*
    @PostMapping(value = "{employeeId}/attach/businessProcess/{businessProcessId}")
    ResponseEntity<String> attachBusinessProcess(@PathVariable long employeeId, @PathVariable long businessProcessId) {
        Session session = sessionFactory.openSession();
        var employee = session.get(Employee.class, employeeId);
        if(employee == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("No employee with id" + employeeId);
        }

        var businessProcess = session.get(BusinessProcess.class, businessProcessId);
        if(businessProcess == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("No business process with id" + employeeId);
        }
        Transaction tx = session.beginTransaction();
        businessProcess.contact = employee;
        employee.businessProcesses.add(businessProcess);
        tx.commit();
        session.close();

        return ResponseEntity.status(HttpStatus.CREATED).body("Successfully attached process");
    }
     */
}
