package com.skica.spvbpisvsportalbackend.controller.unused;

import com.skica.spvbpisvsportalbackend.entity.unused.Role;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

@Controller
@RequestMapping("/svc/role")
public class RoleController {
    @Autowired
    private SessionFactory sessionFactory;

    @GetMapping({"/view/{entityId}"})
    public String view(@PathVariable("entityId") int entityId, Model model) {
        try (var session = sessionFactory.openSession()) {
            model.addAttribute("role", session.get(Role.class, entityId));
        }
        return "view/role";
    }

    @PostMapping(value="/create")
    public String save(
            @ModelAttribute("role") final Role entity, final BindingResult bindingResult, final ModelMap model) {
        if (bindingResult.hasErrors()) {
            return "/";
        }
        try (var session = sessionFactory.openSession()) {
            var tx = session.beginTransaction();
            session.persist(entity);
            tx.commit();
        }
        model.clear();
        return "redirect:/view/role/" + entity.name;
    }

    @PostMapping(value = "{roleId}/clear")
    ResponseEntity<String> bindRole(@PathVariable long roleId) {
        Session session = sessionFactory.openSession();
        var role = session.get(Role.class, roleId);
        if(role == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("No role with id" + roleId);
        }

        Transaction tx = session.beginTransaction();
        role.employees.forEach(employee -> {
            employee.roles.remove(role);
            session.merge(employee);
        });
        tx.commit();
        session.close();

        return ResponseEntity.status(HttpStatus.CREATED).body("Successfully unbound");
    }
}
