package com.skica.spvbpisvsportalbackend.entity.isms;

import com.skica.spvbpisvsportalbackend.enums.GeneralSecondaryAsset;
import com.skica.spvbpisvsportalbackend.enums.RiskLevel;
import com.skica.spvbpisvsportalbackend.enums.SecurityPrinciple;
import com.skica.spvbpisvsportalbackend.enums.ThreatType;
import jakarta.persistence.*;

import java.util.List;

@Entity
public class IsmsSetup {

    public static final String ID = "ISMS_SETUP_MODEL";
    @Id
    private String id = ID;

    @Embedded
    OfficialInformation officialInformation;

    @Embedded
    FrameworkConditions frameworkConditions;


    @OrderColumn
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    public List<SecurityObjective> securityObjectives;

    String guarantorKIB;

    public List<GeneralSecondaryAsset> secondaryAssets;

    @OrderColumn
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    public List<ThreatAndRisk> threatsAndRisks;
    public RiskLevel acceptableRiskLevel = RiskLevel.LOW;

    public List<SecurityPrinciple> securityPrinciples;

    @Embedded
    public FormalData formalData;

    public List<GeneralSecondaryAsset> getSecondaryAssets() {
        return secondaryAssets;
    }

    public void setSecondaryAssets(List<GeneralSecondaryAsset> secondaryAssets) {
        this.secondaryAssets = secondaryAssets;
    }

    public List<ThreatAndRisk> getThreatsAndRisks() {
        return threatsAndRisks;
    }

    public void setThreatsAndRisks(List<ThreatAndRisk> threatsAndRisks) {
        this.threatsAndRisks = threatsAndRisks;
    }

    public ThreatAndRisk getThreatAndRiskFromThreatType(ThreatType threatType) {
        return threatsAndRisks.stream().filter(threat -> threat.getThreatType() == threatType).findFirst().orElse(null);
    }

    public List<SecurityPrinciple> getSecurityPrinciples() {
        return securityPrinciples;
    }

    public void setSecurityPrinciples(List<SecurityPrinciple> securityPrinciples) {
        this.securityPrinciples = securityPrinciples;
    }

    public FormalData getFormalData() {
        return formalData;
    }

    public void setFormalData(FormalData formalData) {
        this.formalData = formalData;
    }

    public String getGuarantorKIB() {
        return guarantorKIB;
    }

    public void setGuarantorKIB(String guarantorKIB) {
        this.guarantorKIB = guarantorKIB;
    }

    public RiskLevel getAcceptableRiskLevel() {
        return acceptableRiskLevel;
    }

    public void setAcceptableRiskLevel(RiskLevel acceptableRiskLevel) {
        this.acceptableRiskLevel = acceptableRiskLevel;
    }

    public OfficialInformation getOfficialInformation() {
        if(officialInformation == null) {
            this.officialInformation = new OfficialInformation();
        }
        return officialInformation;
    }

    public void setOfficialInformation(OfficialInformation officialInformation) {
        this.officialInformation = officialInformation;
    }

    public FrameworkConditions getFrameworkConditions() {
        if(frameworkConditions == null) {
            this.frameworkConditions = new FrameworkConditions();
        }
        return frameworkConditions;
    }

    public void setFrameworkConditions(FrameworkConditions frameworkConditions) {
        this.frameworkConditions = frameworkConditions;
    }

    public List<SecurityObjective> getSecurityObjectives() {
        return securityObjectives;
    }

    public void setSecurityObjectives(List<SecurityObjective> securityObjectives) {
        this.securityObjectives = securityObjectives;
    }
}
