package com.skica.spvbpisvsportalbackend.entity.isms;

import com.skica.spvbpisvsportalbackend.enums.ThreatImpact;
import com.skica.spvbpisvsportalbackend.enums.RiskLevel;
import com.skica.spvbpisvsportalbackend.enums.ThreatProbability;
import com.skica.spvbpisvsportalbackend.enums.ThreatType;
import jakarta.persistence.Entity;
import jakarta.persistence.Id;

import java.util.Map;

@Entity
public class ThreatAndRisk {

    @Id
    public ThreatType threatType;
    public RiskLevel riskLevel;

    public ThreatProbability riskProbability = ThreatProbability.NEGLIGIBLE;
    public ThreatImpact riskImpact = ThreatImpact.NEGLIGIBLE;

    private static Map<ThreatProbability, Map<ThreatImpact, RiskLevel>> riskLevelMatrix = Map.of(
            ThreatProbability.NEGLIGIBLE, Map.of(
                    ThreatImpact.NEGLIGIBLE, RiskLevel.LOW,
                    ThreatImpact.CONSIDERABLE, RiskLevel.LOW,
                    ThreatImpact.CRITICAL, RiskLevel.MEDIUM),
            ThreatProbability.OFTEN, Map.of(
                    ThreatImpact.NEGLIGIBLE, RiskLevel.LOW,
                    ThreatImpact.CONSIDERABLE, RiskLevel.MEDIUM,
                    ThreatImpact.CRITICAL, RiskLevel.HIGH),
            ThreatProbability.VERY_OFTEN, Map.of(
                    ThreatImpact.NEGLIGIBLE, RiskLevel.LOW,
                    ThreatImpact.CONSIDERABLE, RiskLevel.HIGH,
                    ThreatImpact.CRITICAL, RiskLevel.HIGH)
    );

    public ThreatAndRisk() {}
    public ThreatAndRisk(ThreatType type) {
        this.threatType = type;
    }
    public ThreatType getThreatType() {
        return threatType;
    }

    public void setThreatType(ThreatType threatType) {
        this.threatType = threatType;
    }

    public RiskLevel getThreatLevel() {
        return riskLevel;
    }

    public void setThreatLevel(RiskLevel riskLevel) {
        this.riskLevel = riskLevel;
    }

    public ThreatProbability getRiskProbability() {
        return riskProbability;
    }

    public void setRiskProbability(ThreatProbability riskProbability) {
        this.riskProbability = riskProbability;
    }

    public ThreatImpact getRiskImpact() {
        return riskImpact;
    }

    public void setRiskImpact(ThreatImpact riskImpact) {
        this.riskImpact = riskImpact;
    }

    public void calculateThreatLevel(ThreatProbability riskProbability, ThreatImpact riskImpact) {
        this.riskProbability = riskProbability;
        this.riskImpact = riskImpact;
        this.riskLevel = riskLevelMatrix.get(riskProbability).get(riskImpact);
    }
}
