package com.skica.spvbpisvsportalbackend.workflow.controller;

import com.skica.spvbpisvsportalbackend.entity.isms.FormalData;
import com.skica.spvbpisvsportalbackend.entity.isms.ThreatAndRisk;
import com.skica.spvbpisvsportalbackend.entity.isms.OfficialInformation;
import com.skica.spvbpisvsportalbackend.entity.isms.SecurityObjective;
import com.skica.spvbpisvsportalbackend.enums.*;
import com.skica.spvbpisvsportalbackend.workflow.service.IsmsSetupService;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Controller
@RequestMapping("/svc/isms-setup")
public class IsmsSetupControler {

    @Autowired
    private IsmsSetupService ismsSetupService;

    @PostMapping("/official-information")
    public ResponseEntity<Void> setOfficialInformation(
        @RequestBody OfficialInformation officialInformation) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        ismsSetupModel.setOfficialInformation(officialInformation);
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return ResponseEntity.noContent().build();
    }

    @PostMapping("/framework-conditions/{text}")
    public ResponseEntity<Void> save(@PathVariable("text") final String text, HttpServletRequest request) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        var frameworkConditions = ismsSetupModel.getFrameworkConditions();
        switch (text) {
            case "organisation-goals" ->
                    frameworkConditions.setOrganisationGoals(request.getParameter("organisationGoals"));
            case "organisation-structure" ->
                    frameworkConditions.setOrganisationStructure(request.getParameter("organisationStructure"));
            case "organisation-third-parties" ->
                    frameworkConditions.setOrganisationThirdParties(request.getParameter("organisationThirdParties"));
            case "organisation-strategic-context" ->
                    frameworkConditions.setOrganisationStrategicContext(request.getParameter("organisationStrategicContext"));
            default -> {
                return ResponseEntity.notFound().build();
            }
        }
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return ResponseEntity.noContent().build();
    }
    @PostMapping(value="/security-objective/create")
    public String save(
            @ModelAttribute("createGeneralSecurityObjective") final SecurityObjective entity,
            @RequestParam(value = "selectedBasicProtectionValues", required = false) final List<Integer> selectedBasicProtectionValues,
            final BindingResult bindingResult,
            final ModelMap model,
            final HttpServletRequest httpServletRequest) {
        if (bindingResult.hasErrors()) {
            return "/";
        }

        if(selectedBasicProtectionValues != null) {
            entity.basicProtectionValues = selectedBasicProtectionValues.stream().map(s -> BasicProtectionValue.values()[s]).toList();
        }
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        ismsSetupModel.getSecurityObjectives().add(entity);
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        model.clear();
        return "redirect:" + httpServletRequest.getHeader(HttpHeaders.REFERER);
    }

    @PostMapping({"/security-objective/delete/{entityId}"})
    public String delete(@PathVariable("entityId") int entityId, final HttpServletRequest httpServletRequest) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        ismsSetupModel.getSecurityObjectives().removeIf(gsp-> gsp.getId() == entityId);
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return "redirect:" + httpServletRequest.getHeader(HttpHeaders.REFERER);
    }

    @PostMapping("/guarantor-kib")
    public ResponseEntity<Void> setOrganisationName(
            @RequestParam(value = "guarantorKIB", required = false) String guarantorKIB) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        ismsSetupModel.setGuarantorKIB(guarantorKIB);
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return ResponseEntity.noContent().build();
    }

    @PostMapping("/general-secondary-assets")
    public ResponseEntity<Void> setMain(@RequestParam(value = "selectedGeneralSecondaryAssets", required = false) List<Integer> selectedGeneralSecondaryAssets) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        ismsSetupModel.secondaryAssets = selectedGeneralSecondaryAssets == null
                ? List.of()
                : selectedGeneralSecondaryAssets.stream().map(sgsa -> GeneralSecondaryAsset.values()[sgsa]).toList();
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return ResponseEntity.noContent().build();
    }

    @PostMapping("/select-general-threats")
    public ResponseEntity<Void> setGeneralThreats(@RequestParam(value = "selectedGeneralThreatTypes", required = false) List<Integer> selectedGeneralThreatTypes) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();

        ismsSetupModel.threatsAndRisks = selectedGeneralThreatTypes == null
                ? new ArrayList<>()
                // don't know why but when immutable collection is passed, it crashes
                : selectedGeneralThreatTypes.stream().map(sgt -> new ThreatAndRisk(ThreatType.values()[sgt])).collect(Collectors.toCollection(ArrayList::new));
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return ResponseEntity.noContent().build();
    }

    @PostMapping("/update-risks")
    public String updateThreatsAndRisks(
            @RequestParam(value = "riskProbability") List<Integer> riskProbability,
            @RequestParam(value = "riskImpact") List<Integer> riskImpact,
            final HttpServletRequest httpServletRequest) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();

        int i = 0;
        for(var threatAndRisk : ismsSetupModel.threatsAndRisks) {
            threatAndRisk.calculateThreatLevel(ThreatProbability.values()[riskProbability.get(i)], ThreatImpact.values()[riskImpact.get(i)]);
            ++i;
        }
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return "redirect:" + httpServletRequest.getHeader(HttpHeaders.REFERER);
    }

    @PostMapping("/acceptable-risk-level")
    public ResponseEntity<Void> setAcceptableRiskLevel(
            @RequestParam(value = "acceptableRiskLevel", required = false) Integer acceptableRiskLevel) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        if(acceptableRiskLevel == null) {
            ismsSetupModel.acceptableRiskLevel = null;
        } else {
            ismsSetupModel.acceptableRiskLevel = RiskLevel.values()[acceptableRiskLevel];
        }
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return ResponseEntity.noContent().build();
    }
    @PostMapping("/select-security-principles")
    public ResponseEntity<Void> selectSecurityPrinciples(@RequestParam(value = "selectedSecurityPrinciples", required = false) List<Integer> selectedSecurityPrinciples) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        ismsSetupModel.securityPrinciples = selectedSecurityPrinciples == null
                ? List.of()
                : selectedSecurityPrinciples.stream().map(ssp -> SecurityPrinciple.values()[ssp]).toList();
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return ResponseEntity.noContent().build();
    }
    @PostMapping("/formal-data")
    public ResponseEntity<Void> selectSecurityPrinciples(@RequestBody FormalData formalData) {
        var ismsSetupModel = ismsSetupService.getIsmsSetupModel();
        ismsSetupModel.formalData = formalData;
        ismsSetupService.saveIsmsSetupModel(ismsSetupModel);

        return ResponseEntity.noContent().build();
    }
}